Package: alabaster.base
Title: Save Bioconductor Objects to File
Version: 1.12.1
Date: 2026-06-30
Authors@R: person("Aaron", "Lun", role=c("aut", "cre"), email="infinite.monkeys.with.keyboards@gmail.com")
License: MIT + file LICENSE
Description: 
    Save Bioconductor data structures into file artifacts, and load them back into memory.
    This is a more robust and portable alternative to serialization of such objects into RDS files.
    Each artifact is associated with metadata for further interpretation;
    downstream applications can enrich this metadata with context-specific properties.
Imports: alabaster.schemas, methods, utils, S4Vectors, rhdf5 (>=
        2.47.6), jsonlite, jsonvalidate, Rcpp
Suggests: BiocStyle, rmarkdown, knitr, testthat, digest, Matrix,
        alabaster.matrix
LinkingTo: Rcpp, assorthead (>= 1.1.2), Rhdf5lib
VignetteBuilder: knitr
SystemRequirements: C++17, GNU make
RoxygenNote: 7.3.3
Encoding: UTF-8
biocViews: DataRepresentation, DataImport
URL: https://github.com/ArtifactDB/alabaster.base
BugReports: https://github.com/ArtifactDB/alabaster.base/issues
git_url: https://git.bioconductor.org/packages/alabaster.base
git_branch: RELEASE_3_23
git_last_commit: 4898f21
git_last_commit_date: 2026-06-29
Repository: Bioconductor 3.23
Date/Publication: 2026-07-02
NeedsCompilation: yes
Packaged: 2026-07-02 20:19:13 UTC; biocbuild
Author: Aaron Lun [aut, cre]
Maintainer: Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>
