# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

check_csv <- function(path, is_compressed, parallel) {
    .Call(`_alabaster_base_check_csv`, path, is_compressed, parallel)
}

check_list_hdf5 <- function(file, name, num_external) {
    .Call(`_alabaster_base_check_list_hdf5`, file, name, num_external)
}

check_list_json <- function(file, num_external, parallel) {
    .Call(`_alabaster_base_check_list_json`, file, num_external, parallel)
}

any_actually_numeric_na <- function(x) {
    .Call(`_alabaster_base_any_actually_numeric_na`, x)
}

is_actually_numeric_na <- function(x) {
    .Call(`_alabaster_base_is_actually_numeric_na`, x)
}

choose_numeric_missing_placeholder <- function(x) {
    .Call(`_alabaster_base_choose_numeric_missing_placeholder`, x)
}

not_rfc3339 <- function(x) {
    .Call(`_alabaster_base_not_rfc3339`, x)
}

load_csv <- function(path, is_compressed, nrecords, parallel) {
    .Call(`_alabaster_base_load_csv`, path, is_compressed, nrecords, parallel)
}

load_list_hdf5 <- function(file, name, obj) {
    .Call(`_alabaster_base_load_list_hdf5`, file, name, obj)
}

load_list_json <- function(file, obj, parallel) {
    .Call(`_alabaster_base_load_list_json`, file, obj, parallel)
}

validate <- function(path, metadata) {
    .Call(`_alabaster_base_validate`, path, metadata)
}

register_validate_function <- function(type, fun, existing) {
    .Call(`_alabaster_base_register_validate_function`, type, fun, existing)
}

deregister_validate_function <- function(type) {
    .Call(`_alabaster_base_deregister_validate_function`, type)
}

register_height_function <- function(type, fun, existing) {
    .Call(`_alabaster_base_register_height_function`, type, fun, existing)
}

deregister_height_function <- function(type) {
    .Call(`_alabaster_base_deregister_height_function`, type)
}

register_dimensions_function <- function(type, fun, existing) {
    .Call(`_alabaster_base_register_dimensions_function`, type, fun, existing)
}

deregister_dimensions_function <- function(type) {
    .Call(`_alabaster_base_deregister_dimensions_function`, type)
}

register_any_duplicated <- function(set) {
    .Call(`_alabaster_base_register_any_duplicated`, set)
}

register_satisfies_interface <- function(type, interface) {
    .Call(`_alabaster_base_register_satisfies_interface`, type, interface)
}

deregister_satisfies_interface <- function(type, interface) {
    .Call(`_alabaster_base_deregister_satisfies_interface`, type, interface)
}

register_derived_from <- function(type, parent) {
    .Call(`_alabaster_base_register_derived_from`, type, parent)
}

deregister_derived_from <- function(type, parent) {
    .Call(`_alabaster_base_deregister_derived_from`, type, parent)
}

use_vls <- function(x) {
    .Call(`_alabaster_base_use_vls`, x)
}

dump_vls <- function(file, group, pointers, heap, x, raw_dims, raw_chunks, compress, scalar, buffer_size) {
    .Call(`_alabaster_base_dump_vls`, file, group, pointers, heap, x, raw_dims, raw_chunks, compress, scalar, buffer_size)
}

parse_vls <- function(file, pointers, heap, buffer_size, placeholder, native) {
    .Call(`_alabaster_base_parse_vls`, file, pointers, heap, buffer_size, placeholder, native)
}

